@@ -1,6 +1,7 @@
 /* homedir.c - Setup the home directory.
  * Copyright (C) 2004, 2006, 2007, 2010 Free Software Foundation, Inc.
  * Copyright (C) 2013, 2016 Werner Koch
+ * Copyright (C) 2021 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -26,6 +27,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: (LGPL-3.0-or-later OR GPL-2.0-or-later)
  */
 
 #include <config.h>
@@ -61,6 +63,7 @@
 
 #include "util.h"
 #include "sysutils.h"
+#include "i18n.h"
 #include "zb32.h"
 
 /* The GnuPG homedir.  This is only accessed by the functions
@@ -73,15 +76,15 @@ static byte non_default_homedir;
 
 #ifdef HAVE_W32_SYSTEM
 /* A flag used to indicate that a control file for gpgconf has been
-   detected.  Under Windows the presence of this file indicates a
-   portable installations and triggers several changes:
-
-   - The GNUGHOME directory is fixed relative to installation
-     directory.  All other means to set the home directory are ignore.
-
-   - All registry variables will be ignored.
-
-   This flag is not used on Unix systems.
+ * detected.  Under Windows the presence of this file indicates a
+ * portable installations and triggers several changes:
+ *
+ * - The GNUGHOME directory is fixed relative to installation
+ *   directory.  All other means to set the home directory are ignored.
+ *
+ * - All registry variables will be ignored.
+ *
+ * This flag is not used on Unix systems.
  */
 static byte w32_portable_app;
 #endif /*HAVE_W32_SYSTEM*/
@@ -117,14 +120,16 @@ w32_try_mkdir (const char *dir)
 #endif
 
 
-/* This is a helper function to load a Windows function from either of
-   one DLLs. */
+/* This is a helper function to load and call a Windows function from
+ * either of one DLLs.  On success an UTF-8 file name is returned.
+ * ERRNO is _not_ set on error.  */
 #ifdef HAVE_W32_SYSTEM
-static HRESULT
-w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
+static char *
+w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d)
 {
   static int initialized;
-  static HRESULT (WINAPI * func)(HWND,int,HANDLE,DWORD,LPSTR);
+  static HRESULT (WINAPI * func)(HWND,int,HANDLE,DWORD,LPWSTR);
+  wchar_t wfname[MAX_PATH];
 
   if (!initialized)
     {
@@ -139,7 +144,7 @@ w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
           handle = dlopen (dllnames[i], RTLD_LAZY);
           if (handle)
             {
-              func = dlsym (handle, "SHGetFolderPathA");
+              func = dlsym (handle, "SHGetFolderPathW");
               if (!func)
                 {
                   dlclose (handle);
@@ -149,10 +154,10 @@ w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
         }
     }
 
-  if (func)
-    return func (a,b,c,d,e);
+  if (func && func (a,b,c,d,wfname) >= 0)
+    return wchar_to_utf8 (wfname);
   else
-    return -1;
+    return NULL;
 }
 #endif /*HAVE_W32_SYSTEM*/
 
@@ -163,7 +168,7 @@ is_gnupg_default_homedir (const char *dir)
 {
   int result;
   char *a = make_absfilename (dir, NULL);
-  char *b = make_absfilename (GNUPG_DEFAULT_HOMEDIR, NULL);
+  char *b = make_absfilename (standard_homedir (), NULL);
   result = !compare_filenames (a, b);
   xfree (b);
   xfree (a);
@@ -171,6 +176,62 @@ is_gnupg_default_homedir (const char *dir)
 }
 
 
+/* Helper to remove trailing slashes from NEWDIR.  Return a new
+ * allocated string if that has been done or NULL if there are no
+ * slashes to remove.  Also inserts a missing slash after a Windows
+ * drive letter.  */
+static char *
+copy_dir_with_fixup (const char *newdir)
+{
+  char *result = NULL;
+  char *p;
+
+  if (!*newdir)
+    return NULL;
+
+#ifdef HAVE_W32_SYSTEM
+  if (newdir[0] && newdir[1] == ':'
+      && !(newdir[2] == '/' || newdir[2] == '\\'))
+    {
+      /* Drive letter with missing leading slash.  */
+      p = result = xmalloc (strlen (newdir) + 1 + 1);
+      *p++ = newdir[0];
+      *p++ = newdir[1];
+      *p++ = '\\';
+      strcpy (p, newdir+2);
+
+      /* Remove trailing slashes.  */
+      p = result + strlen (result) - 1;
+      while (p > result+2 && (*p == '/' || *p == '\\'))
+        *p-- = 0;
+    }
+  else if (newdir[strlen (newdir)-1] == '/'
+           || newdir[strlen (newdir)-1] == '\\' )
+    {
+      result = xstrdup (newdir);
+      p = result + strlen (result) - 1;
+      while (p > result
+             && (*p == '/' || *p == '\\')
+             && (p-1 > result && p[-1] != ':')) /* We keep "c:/". */
+        *p-- = 0;
+    }
+
+#else /*!HAVE_W32_SYSTEM*/
+
+  if (newdir[strlen (newdir)-1] == '/')
+    {
+      result = xstrdup (newdir);
+      p = result + strlen (result) - 1;
+      while (p > result && *p == '/')
+        *p-- = 0;
+    }
+
+#endif /*!HAVE_W32_SYSTEM*/
+
+  return result;
+}
+
+
 /* Get the standard home directory.  In general this function should
    not be used as it does not consider a registry value (under W32) or
    the GNUPGHOME environment variable.  It is better to use
@@ -192,25 +253,17 @@ standard_homedir (void)
         }
       else
         {
-          char path[MAX_PATH];
+          char *path;
 
-          /* It might be better to use LOCAL_APPDATA because this is
-             defined as "non roaming" and thus more likely to be kept
-             locally.  For private keys this is desired.  However,
-             given that many users copy private keys anyway forth and
-             back, using a system roaming services might be better
-             than to let them do it manually.  A security conscious
-             user will anyway use the registry entry to have better
-             control.  */
-          if (w32_shgetfolderpath (NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE,
-                                   NULL, 0, path) >= 0)
+          path = w32_shgetfolderpath (NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE,
+                                      NULL, 0);
+          if (path)
             {
-              char *tmp = xmalloc (strlen (path) + 6 +1);
-              strcpy (stpcpy (tmp, path), "\\gnupg");
-              dir = tmp;
+              dir = xstrconcat (path, "\\gnupg", NULL);
+              xfree (path);
 
               /* Try to create the directory if it does not yet exists.  */
-              if (access (dir, F_OK))
+              if (gnupg_access (dir, F_OK))
                 w32_try_mkdir (dir);
             }
           else
@@ -249,6 +302,9 @@ default_homedir (void)
             {
               char *tmp, *p;
 
+              /* This is deprecated; gpgconf --list-dirs prints a
+               * warning if the homedir has been taken from the
+               * registry.  */
               tmp = read_w32_registry_string (NULL,
                                               GNUPG_REGISTRY_DIR,
                                               "HomeDir");
@@ -278,18 +334,11 @@ default_homedir (void)
     dir = GNUPG_DEFAULT_HOMEDIR;
   else
     {
-      /* Strip trailing slashes if any.  */
-      if (dir[strlen (dir)-1] == '/')
-        {
-          char *tmp, *p;
+      char *p;
 
-          tmp = xstrdup (dir);
-          p = tmp + strlen (tmp) - 1;
-          while (p > tmp && *p == '/')
-            *p-- = 0;
-
-          dir = tmp;
-        }
+      p = copy_dir_with_fixup (dir);
+      if (p)
+        dir = p;
 
       if (!is_gnupg_default_homedir (dir))
         non_default_homedir = 1;
@@ -308,10 +357,10 @@ check_portable_app (const char *dir)
   char *fname;
 
   fname = xstrconcat (dir, DIRSEP_S "gpgconf.exe", NULL);
-  if (!access (fname, F_OK))
+  if (!gnupg_access (fname, F_OK))
     {
       strcpy (fname + strlen (fname) - 3, "ctl");
-      if (!access (fname, F_OK))
+      if (!gnupg_access (fname, F_OK))
         {
           /* gpgconf.ctl file found.  Record this fact.  */
           w32_portable_app = 1;
@@ -388,7 +437,7 @@ w32_commondir (void)
   if (!dir)
     {
       const char *rdir;
-      char path[MAX_PATH];
+      char *path;
 
       /* Make sure that w32_rootdir has been called so that we are
          able to check the portable application flag.  The common dir
@@ -398,19 +447,17 @@ w32_commondir (void)
       if (w32_portable_app)
         return rdir;
 
-      if (w32_shgetfolderpath (NULL, CSIDL_COMMON_APPDATA,
-                               NULL, 0, path) >= 0)
+      path = w32_shgetfolderpath (NULL, CSIDL_COMMON_APPDATA, NULL, 0);
+      if (path)
         {
-          char *tmp = xmalloc (strlen (path) + 4 +1);
-          strcpy (stpcpy (tmp, path), "\\GNU");
-          dir = tmp;
+          dir = xstrconcat (path, "\\GNU", NULL);
           /* No auto create of the directory.  Either the installer or
-             the admin has to create these directories.  */
+           * the admin has to create these directories.  */
         }
       else
         {
-          /* Ooops: Not defined - probably an old Windows version.
-             Use the installation directory instead.  */
+          /* Folder not found or defined - probably an old Windows
+           * version.  Use the installation directory instead.  */
           dir = xstrdup (rdir);
         }
     }
@@ -432,28 +479,10 @@ gnupg_set_homedir (const char *newdir)
     newdir = default_homedir ();
   else
     {
-      /* Remove trailing slashes from NEWSDIR.  */
-      if (newdir[strlen (newdir)-1] == '/'
-#ifdef HAVE_W32_SYSTEM
-          || newdir[strlen (newdir)-1] == '\\'
-#endif
-          )
-        {
-          char *p;
+      tmp = copy_dir_with_fixup (newdir);
+      if (tmp)
+        newdir = tmp;
 
-          tmp = xstrdup (newdir);
-          p = tmp + strlen (tmp) - 1;
-          while (p > tmp
-                 && (*p == '/'
-#ifdef HAVE_W32_SYSTEM
-                     || *p == '\\'
-#endif
-                     )
-                 )
-            *p-- = 0;
-
-          newdir = tmp;
-        }
       if (!is_gnupg_default_homedir (newdir))
         non_default_homedir = 1;
     }
@@ -463,6 +492,38 @@ gnupg_set_homedir (const char *newdir)
 }
 
 
+/* Create the homedir directory only if the supplied directory name is
+ * the same as the default one.  This way we avoid to create arbitrary
+ * directories when a non-default home directory is used.  To cope
+ * with HOME, we do compare only the suffix if we see that the default
+ * homedir does start with a tilde.  If the mkdir fails the function
+ * terminates the process.  If QUIET is set not diagnostic is printed
+ * on homedir creation.  */
+void
+gnupg_maybe_make_homedir (const char *fname, int quiet)
+{
+  const char *defhome = standard_homedir ();
+
+  if (
+#ifdef HAVE_W32_SYSTEM
+      ( !compare_filenames (fname, defhome) )
+#else
+      ( *defhome == '~'
+        && (strlen(fname) >= strlen (defhome+1)
+            && !strcmp(fname+strlen(fname)-strlen(defhome+1), defhome+1 ) ))
+      || (*defhome != '~'  && !compare_filenames( fname, defhome ) )
+#endif
+      )
+    {
+      if (gnupg_mkdir (fname, "-rwx"))
+        log_fatal ( _("can't create directory '%s': %s\n"),
+                    fname, strerror(errno) );
+      else if (!quiet )
+        log_info ( _("directory '%s' created\n"), fname );
+    }
+}
+
+
 /* Return the homedir.  The returned string is valid until another
  * gnupg-set-homedir call.  This is always an absolute directory name.
  * The function replaces the former global var opt.homedir.  */
@@ -531,8 +592,117 @@ gnupg_daemon_rootdir (void)
 char *
 _gnupg_socketdir_internal (int skip_checks, unsigned *r_info)
 {
-#if defined(HAVE_W32_SYSTEM) || !defined(HAVE_STAT)
+#if defined(HAVE_W32_SYSTEM)
+  char *name;
 
+  (void)skip_checks;
+
+  *r_info = 0;
+
+  /* First make sure that non_default_homedir and w32_portable_app can
+   * be set.  */
+  gnupg_homedir ();
+
+  if (w32_portable_app)
+    {
+      name = xstrconcat (w32_rootdir (), DIRSEP_S, "gnupg", NULL);
+    }
+  else
+    {
+      char *path;
+
+      path = w32_shgetfolderpath (NULL,
+                                  CSIDL_LOCAL_APPDATA|CSIDL_FLAG_CREATE,
+                                  NULL, 0);
+      if (path)
+        {
+          name = xstrconcat (path, "\\gnupg", NULL);
+          xfree (path);
+          if (gnupg_access (name, F_OK))
+            w32_try_mkdir (name);
+        }
+      else
+        {
+          name = xstrdup (gnupg_homedir ());
+        }
+    }
+
+  /* If a non default homedir is used, we check whether an
+   * corresponding sub directory below the socket dir is available
+   * and use that.  We hash the non default homedir to keep the new
+   * subdir short enough.  */
+  if (non_default_homedir)
+    {
+      char sha1buf[20];
+      struct stat sb;
+      char *suffix;
+      char *p;
+
+      *r_info |= 32; /* Testing subdir.  */
+
+      /* Canonicalize the name to avoid problems with mixed case
+       * names.  Note that we use only 10 bytes of the hash because on
+       * Windows the account name is also part of the name.  */
+      suffix = ascii_strlwr (xstrdup (gnupg_homedir ()));
+      for (p=suffix; *p; p++)
+        if ( *p == '\\')
+          *p = '/';
+      gcry_md_hash_buffer (GCRY_MD_SHA1, sha1buf, suffix, strlen (suffix));
+      xfree (suffix);
+      suffix = zb32_encode (sha1buf, 8*10);
+      if (!suffix)
+        {
+          *r_info |= 1; /* Out of core etc. */
+          goto leave_w32;
+        }
+      p = xstrconcat (name, "\\d.", suffix, NULL);
+      xfree (suffix);
+      xfree (name);
+      name = p;
+
+      /* Stat that directory and check constraints.
+       * The command
+       *    gpgconf --remove-socketdir
+       * can be used to remove that directory.  */
+      if (gnupg_stat (name, &sb))
+        {
+          if (errno != ENOENT)
+            *r_info |= 1; /* stat failed. */
+          else if (!skip_checks)
+            {
+              /* Try to create the directory and check again.  */
+              if (gnupg_mkdir (name, "-rwx"))
+                *r_info |= 16; /* mkdir failed.  */
+              else if (gnupg_stat (name, &sb))
+                {
+                  if (errno != ENOENT)
+                    *r_info |= 1; /* stat failed. */
+                  else
+                    *r_info |= 64; /* Subdir does not exist.  */
+                }
+              else
+                goto leave_w32; /* Success!  */
+            }
+          else
+            *r_info |= 64; /* Subdir does not exist.  */
+          if (!skip_checks)
+            {
+              xfree (name);
+              name = NULL;
+              goto leave_w32;
+            }
+        }
+    }
+
+ leave_w32:
+  /* If nothing works - fall back to the homedir.  */
+  if (!name)
+    {
+      *r_info |= 128; /* Fallback.  */
+      name = xstrdup (gnupg_homedir ());
+    }
+
+#elif !defined(HAVE_STAT)
   char *name;
 
   (void)skip_checks;
@@ -554,7 +724,7 @@ _gnupg_socketdir_internal (int skip_checks, unsigned *r_info)
   };
   int i;
   struct stat sb;
-  char prefix[13 + 1 + 20 + 6 + 1];
+  char prefix[19 + 1 + 20 + 6 + 1];
   const char *s;
   char *name = NULL;
 
@@ -846,71 +1016,18 @@ gnupg_localedir (void)
 }
 
 
-/* Return the name of the cache directory.  The name is allocated in a
-   static area on the first use.  Windows only: If the directory does
-   not exist it is created.  */
+/* Return the standard socket name used by gpg-agent.  */
 const char *
-gnupg_cachedir (void)
+gpg_agent_socket_name (void)
 {
-#ifdef HAVE_W32_SYSTEM
-  static const char *dir;
+  static char *name;
 
-  if (!dir)
+  if (!name)
     {
-      const char *rdir;
-
-      rdir = w32_rootdir ();
-      if (w32_portable_app)
-        {
-          dir = xstrconcat (rdir,
-                            DIRSEP_S, "var",
-                            DIRSEP_S, "cache",
-                            DIRSEP_S, "gnupg", NULL);
-        }
-      else
-        {
-          char path[MAX_PATH];
-          const char *s1[] = { "GNU", "cache", "gnupg", NULL };
-          int s1_len;
-          const char **comp;
-
-          s1_len = 0;
-          for (comp = s1; *comp; comp++)
-            s1_len += 1 + strlen (*comp);
-
-          if (w32_shgetfolderpath (NULL, CSIDL_LOCAL_APPDATA|CSIDL_FLAG_CREATE,
-                                   NULL, 0, path) >= 0)
-            {
-              char *tmp = xmalloc (strlen (path) + s1_len + 1);
-              char *p;
-
-              p = stpcpy (tmp, path);
-              for (comp = s1; *comp; comp++)
-                {
-                  p = stpcpy (p, "\\");
-                  p = stpcpy (p, *comp);
-
-                  if (access (tmp, F_OK))
-                    w32_try_mkdir (tmp);
-                }
-
-              dir = tmp;
-            }
-          else
-            {
-              dir = "c:\\temp\\cache\\gnupg";
-#ifdef HAVE_W32CE_SYSTEM
-              dir += 2;
-              w32_try_mkdir ("\\temp\\cache");
-              w32_try_mkdir ("\\temp\\cache\\gnupg");
-#endif
-            }
-        }
+      name = make_filename (gnupg_socketdir (), GPG_AGENT_SOCK_NAME, NULL);
+      gpgrt_annotate_leaked_object (name);
     }
-  return dir;
-#else /*!HAVE_W32_SYSTEM*/
-  return GNUPG_LOCALSTATEDIR "/cache/" PACKAGE_NAME;
-#endif /*!HAVE_W32_SYSTEM*/
+  return name;
 }
 
 
@@ -942,6 +1059,8 @@ get_default_pinentry_name (int reset)
     /* Try Gpg4win directory (with bin and without.) */
     { w32_rootdir, "\\..\\Gpg4win\\bin\\pinentry.exe" },
     { w32_rootdir, "\\..\\Gpg4win\\pinentry.exe" },
+    /* Try a pinentry in a dir above us */
+    { w32_rootdir, "\\..\\bin\\pinentry.exe" },
     /* Try old Gpgwin directory.  */
     { w32_rootdir, "\\..\\GNU\\GnuPG\\pinentry.exe" },
     /* Try a Pinentry from the common GNU dir.  */
@@ -968,7 +1087,7 @@ get_default_pinentry_name (int reset)
           char *name2;
 
           name2 = xstrconcat (names[i].rfnc (), names[i].name, NULL);
-          if (!access (name2, F_OK))
+          if (!gnupg_access (name2, F_OK))
             {
               /* Use that pinentry.  */
               xfree (name);
