@@ -562,6 +562,12 @@ tdbio_update_version_record (ctrl_t ctrl)
 {
   TRUSTREC rec;
   int rc;
+  int opt_tm;
+
+  /* Never store a TOFU trust model in the trustdb.  Use PGP instead.  */
+  opt_tm = opt.trust_model;
+  if (opt_tm == TM_TOFU || opt_tm == TM_TOFU_PGP)
+    opt_tm = TM_PGP;
 
   memset (&rec, 0, sizeof rec);
 
@@ -572,7 +578,7 @@ tdbio_update_version_record (ctrl_t ctrl)
       rec.r.ver.marginals   = opt.marginals_needed;
       rec.r.ver.completes   = opt.completes_needed;
       rec.r.ver.cert_depth  = opt.max_cert_depth;
-      rec.r.ver.trust_model = opt.trust_model;
+      rec.r.ver.trust_model = opt_tm;
       rec.r.ver.min_cert_level = opt.min_cert_level;
       rc = tdbio_write_record (ctrl, &rec);
     }
@@ -591,6 +597,12 @@ create_version_record (ctrl_t ctrl)
 {
   TRUSTREC rec;
   int rc;
+  int opt_tm;
+
+  /* Never store a TOFU trust model in the trustdb.  Use PGP instead.  */
+  opt_tm = opt.trust_model;
+  if (opt_tm == TM_TOFU || opt_tm == TM_TOFU_PGP)
+    opt_tm = TM_PGP;
 
   memset (&rec, 0, sizeof rec);
   rec.r.ver.version     = 3;
@@ -598,8 +610,8 @@ create_version_record (ctrl_t ctrl)
   rec.r.ver.marginals   = opt.marginals_needed;
   rec.r.ver.completes   = opt.completes_needed;
   rec.r.ver.cert_depth  = opt.max_cert_depth;
-  if (opt.trust_model == TM_PGP || opt.trust_model == TM_CLASSIC)
-    rec.r.ver.trust_model = opt.trust_model;
+  if (opt_tm == TM_PGP || opt_tm == TM_CLASSIC)
+    rec.r.ver.trust_model = opt_tm;
   else
     rec.r.ver.trust_model = TM_PGP;
   rec.r.ver.min_cert_level = opt.min_cert_level;
@@ -669,7 +681,7 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
    * trustdb.gpg.  This check is not required in theory, but it helps
    * in practice avoiding costly operations of preparing and taking
    * the lock.  */
-  if (!stat (fname, &statbuf) && statbuf.st_size > 0)
+  if (!gnupg_stat (fname, &statbuf) && statbuf.st_size > 0)
     {
       /* OK, we have the valid trustdb.gpg already.  */
       return 0;
@@ -699,19 +711,21 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
   log_assert (p);
   save_slash = *p;
   *p = 0;
-  if (access (fname, F_OK))
+  if (gnupg_access (fname, F_OK))
     {
       try_make_homedir (fname);
-      if (access (fname, F_OK))
+      if (gnupg_access (fname, F_OK))
         log_fatal (_("%s: directory does not exist!\n"), fname);
     }
   *p = save_slash;
 
   take_write_lock ();
 
-  if (access (fname, R_OK) || stat (fname, &statbuf) || statbuf.st_size == 0)
+  if (gnupg_access (fname, R_OK)
+      || gnupg_stat (fname, &statbuf)
+      || statbuf.st_size == 0)
     {
-      FILE *fp;
+      estream_t fp;
       TRUSTREC rec;
       int rc;
       mode_t oldmask;
@@ -733,13 +747,13 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
           gpg_err_set_errno (EPERM);
         }
       else
-        fp = fopen (fname, "wb");
+        fp = es_fopen (fname, "wb");
       umask(oldmask);
       if (!fp)
         log_fatal (_("can't create '%s': %s\n"), fname, strerror (errno));
-      fclose (fp);
+      es_fclose (fp);
 
-      db_fd = open (db_name, O_RDWR | MY_O_BINARY);
+      db_fd = gnupg_open (db_name, O_RDWR | MY_O_BINARY, 0);
       if (db_fd == -1)
         log_fatal (_("can't open '%s': %s\n"), db_name, strerror (errno));
 
@@ -799,7 +813,7 @@ open_db ()
                  (int)prevrc, (int)GetLastError ());
   }
 #else /*!HAVE_W32CE_SYSTEM*/
-  db_fd = open (db_name, O_RDWR | MY_O_BINARY );
+  db_fd = gnupg_open (db_name, O_RDWR | MY_O_BINARY, 0);
   if (db_fd == -1 && (errno == EACCES
 #ifdef EROFS
                       || errno == EROFS
@@ -807,7 +821,7 @@ open_db ()
                       )
       ) {
       /* Take care of read-only trustdbs.  */
-      db_fd = open (db_name, O_RDONLY | MY_O_BINARY );
+      db_fd = gnupg_open (db_name, O_RDONLY | MY_O_BINARY, 0);
       if (db_fd != -1 && !opt.quiet)
           log_info (_("Note: trustdb not writable\n"));
   }
@@ -883,16 +897,25 @@ tdbio_db_matches_options()
     {
       TRUSTREC vr;
       int rc;
+      int opt_tm, tm;
 
       rc = tdbio_read_record (0, &vr, RECTYPE_VER);
       if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
 		   db_name, gpg_strerror (rc) );
 
+      /* Consider tofu and pgp the same.  */
+      tm = vr.r.ver.trust_model;
+      if (tm == TM_TOFU || tm == TM_TOFU_PGP)
+        tm = TM_PGP;
+      opt_tm  = opt.trust_model;
+      if (opt_tm == TM_TOFU || opt_tm == TM_TOFU_PGP)
+        opt_tm = TM_PGP;
+
       yes_no = vr.r.ver.marginals == opt.marginals_needed
 	&& vr.r.ver.completes == opt.completes_needed
 	&& vr.r.ver.cert_depth == opt.max_cert_depth
-	&& vr.r.ver.trust_model == opt.trust_model
+	&& tm == opt_tm
 	&& vr.r.ver.min_cert_level == opt.min_cert_level;
     }
 
